#pragma once

#include <Eigen/Core>

#include <armarx/navigation/algorithms/types.h>

namespace armarx::navigation::algorithms
{

    class Costmap
    {
    public:
        EIGEN_MAKE_ALIGNED_OPERATOR_NEW

        friend class CostmapBuilder;

        struct Parameters
        {
            // if set to false, distance to obstacles will be computed and not only a binary collision check
            bool binaryGrid{false};

            /// How big each cell is in the uniform grid.
            float cellSize = 100.f;
        };


        using Index = Eigen::Array2i;
        using Position = Eigen::Vector2f;
        using Grid = Eigen::MatrixXf;

        // if 0, the cell is invalid
        using Mask = Eigen::Matrix<bool, Eigen::Dynamic, Eigen::Dynamic>;

        Costmap(const Grid& grid,
                const Parameters& parameters,
                const SceneBounds& sceneBounds,
                const std::optional<Mask>& mask = std::nullopt);

        struct Vertex
        {
            Index index; // row corresponds to y; column corresponds to x
            Position position;
        };

        Position toPosition(const Index& index) const;
        Vertex toVertex(const Position& v) const;

        const Grid& getGrid() const;

        bool isInCollision(const Position& p) const;

        const Parameters& params() const noexcept;

        const SceneBounds& getSceneBounds() const noexcept;

        Index minIndex() const noexcept;

        bool add(const Costmap& other);

    private:
        Grid grid;
        std::optional<Mask> mask;

        const SceneBounds sceneBounds;

        const Parameters parameters;
    };


} // namespace armarx::navigation::algorithms
